import asyncio
import discord
from discord.ext import commands
from discord.ext.commands import Context

from cpl_core.logging import LoggerABC
from cpl_core.configuration import ConfigurationABC

from gismo_core.abc.command_abc import CommandABC
from gismo_core.abc.message_service_abc import MessageServiceABC
from gismo_core.configuration.server_settings import ServerSettings
from modules.base.base_settings import BaseSettings


class BaseCommandService(CommandABC):

    def __init__(
        self,
        logger: LoggerABC,
        config: ConfigurationABC,
        message_service: MessageServiceABC
    ):
        CommandABC.__init__(self)

        self._logger = logger
        self._config = config
        self._message_service = message_service

        self._logger.trace(__name__, f'Loaded command service: {type(self).__name__}')

    @commands.command()
    async def ping(self, ctx: Context):
        self._logger.debug(__name__, f'Received command ping {ctx}')
        await self._message_service.send_ctx_msg(ctx, 'Pong')
        self._logger.trace(__name__, f'Finished ping command')

    @commands.command()
    async def purge(self, ctx: Context):
        self._logger.debug(__name__, f'Received command purge {ctx}')
        settings: BaseSettings = self._config.get_configuration(f'Base_{ctx.guild.id}')
        server_settings: ServerSettings = self._config.get_configuration(f'DSERVER_{ctx.guild.id}')
        
        await self._message_service.send_ctx_msg(ctx, settings.purge_message)
        await asyncio.sleep(server_settings.message_delete_timer)
        await ctx.channel.purge()
        self._logger.trace(__name__, f'Finished purge command')
